#!/usr/bin/python2

# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
### BEGIN LICENSE
# Copyright (C) 2010 manatlan manatlan@gmail.com
# This program is free software: you can redistribute it and/or modify it 
# under the terms of the GNU General Public License version 2, as published 
# by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful, but 
# WITHOUT ANY WARRANTY; without even the implied warranties of 
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR 
# PURPOSE.  See the GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License along 
# with this program.  If not, see <http://www.gnu.org/licenses/>.
### END LICENSE

# see : http://www.manatlan.com/page/autowifi

import sys, os, shutil, gtk, gobject, pynotify, socket 
from subprocess import *
from datetime import datetime

# Check if we are working in the source tree or from the installed
# package and mangle the python path accordingly
if os.path.dirname(sys.argv[0]) != ".":
	if sys.argv[0][0] == "/":
		fullPath = os.path.dirname(sys.argv[0])
	else:
		fullPath = os.getcwd() + "/" + os.path.dirname(sys.argv[0])
else:
	fullPath = os.getcwd()
sys.path.insert(0, os.path.dirname(fullPath))

from lib.AuthentDialog import NewAuthentDialog
from lib.autowificonfig import getdatapath
from lib.pluginloader import Plugins
from lib.netman import NetMan
from lib.config import Config


def notify(s):
	pynotify.init(sys.argv[0])
	pynotify.Notification('AutoWifi', s, os.path.join(getdatapath(),"media","icon.png") ).show()

socket.setdefaulttimeout(5.0) 

class IconApp(gtk.StatusIcon,NetMan):
	
	_GUI = False
	
	def __init__(self,GUI):
		self._GUI = GUI
		if self._GUI:
			gtk.StatusIcon.__init__(self)
		NetMan.__init__(self)

		self._event=None
		self._previousInfo=None
		self._cfg = Config("autowifi")

		localplugins = os.path.join(self._cfg.folder,"plugins")
		realplugins = os.path.join(getdatapath(),"plugins")

		if not os.path.isdir( localplugins ):
			os.makedirs(localplugins)

		self._plugins = Plugins()
		self._plugins.add( localplugins )
		self._plugins.add( realplugins )
		print "Known plugins:\n",self._plugins
		if self._GUI:
			self.connect('popup-menu', self.popup_menu_cb)
			self.set_from_file( os.path.join(getdatapath(),"media","icon.png") )
			self.set_visible(True)
		gobject.timeout_add(2*1000,self.reconnect)  # delay the first start, to be sure that icon is well displayed in systray

	def log(self,o):
		print datetime.now(),o

	def reconnect(self,*a):
		ssid=self.getCurrentSSID()
		self.log("Reconnect to '%s'"%ssid)
		if ssid:
			self.tryconnect(ssid)
		else:
			self.changeInfo("no wifi/ssid","icon.png")
			#self.set_from_file( os.path.join(getdatapath(),"media","icon.png") )

	def on_ssid_changed(self,ssid):
		if self._event is not None:             # to avoid multiple events due to on_ssid_changed qui rerun a tryconnect
			gobject.source_remove(self._event)
			self._event=None

		self.log("SSID changed to '%s'"%ssid)
		self.tryconnect(ssid)

	def tryconnect(self,ssid):
		if self._GUI:
			self.set_from_file( os.path.join(getdatapath(),"media","icon_process.png") )
		go=lambda: self._tryconnect(ssid)
		gobject.idle_add(go)

	def askAuthent( title, login, password ):
		w=NewAuthentDialog()
		w.set_title( title )
		w.login    = login and login.strip() or ""
		w.password = password and password.strip() or ""
		ret=w.run()
		login, password=w.login.strip(),w.password.strip()
		w.destroy()
		if ret == gtk.RESPONSE_OK:
			return login,password
				
	def _tryconnect(self,ssid):
		if ssid:
			plugin = self._plugins.getPluginForSSID(ssid)
			if plugin:
				if hasattr(plugin,"connectWithAuthent") or hasattr(plugin,"connect"):
					login = self._cfg[plugin.id+".login"]
					if not login:
						login =''
					password = self._cfg[plugin.id+".password"]
					if not password:
						password = ''
					(ret,serr) = Popen(['data/plugins/'+plugin.name+'.py', "-u", login, "-p", password ],stdout=PIPE,stderr=PIPE).communicate()
					ret = ret.strip()
				else:
					ret="plugin is broken..."
	
				if ret is True or ret == "True":
					self.changeInfo("Connected (%s)"%ssid,"icon_ok.png")
					#self.set_from_file( os.path.join(getdatapath(),"media","icon_ok.png") )
					self._event=gobject.timeout_add(30*60*1000,self.reconnect) # to keep connection alive !
				elif ret is False or ret == "False":
					self.changeInfo("Authent failed (%s) !" % ssid,"icon_ko.png")
					#self.set_from_file( os.path.join(getdatapath(),"media","icon_ko.png") )
					# get login,password from conf
					login = self._cfg[plugin.id+".login"]
					password = self._cfg[plugin.id+".password"]
					
					if self._GUI:
						# will ask login,password again
						while 1:
							r=askAuthent( "Authent for '%s', sure ?"%ssid,login,password)
							if r:
								login,password = r
								if login and password :
									# save login,password to conf
									self._cfg[plugin.id+".login"]=login
									self._cfg[plugin.id+".password"]=password
									break
							else:
								return
					self._event=gobject.timeout_add(1*1000,self.reconnect)  # to force reconnect as soon as possible
				else:
					self.changeInfo("Error (%s) : %s\nWill retry soon !" % (ssid,str(ret)),"icon_ko.png")
					#self.set_from_file( os.path.join(getdatapath(),"media","icon_ko.png") )
					self._event=gobject.timeout_add(5*1000,self.reconnect)  # to force reconnect as soon as possible
			else:
				# 404 plugin not found ;-)
				self.changeInfo("Unknow ssid (no plugin)","icon.png")
				#self.set_from_file( os.path.join(getdatapath(),"media","icon.png") )
		else:
			# ssid is none
			self.changeInfo("no ssid","icon.png")
			#self.set_from_file( os.path.join(getdatapath(),"media","icon.png") )

	def changeInfo(self,info,icon):
		self.log("INFO:"+info)
		if self._GUI and info != self._previousInfo:
			self.set_tooltip("AutoWifi : %s"%info)
			notify(info)
			self._previousInfo = info
			self.set_from_file( os.path.join(getdatapath(),"media",icon) )

	def quit_cb(self,widget):
		if self._GUI:
			self.set_visible(False)
		gtk.main_quit()

	def popup_menu_cb(self,widget, button, time):
		if button == 3:
			menu = gtk.Menu()
			menuItem = gtk.ImageMenuItem("Force reconnect")
			menuItem.connect('activate', self.reconnect)
			menu.append(menuItem)
			menuItem = gtk.ImageMenuItem(gtk.STOCK_QUIT)
			menuItem.connect('activate', self.quit_cb)
			menu.append(menuItem)
			menu.show_all()
			menu.popup(None, None, None, 3, time)

if __name__ == "__main__":
	import optparse
	parser = optparse.OptionParser(version="%prog %ver")
	parser.add_option("-c", "--cli", action="store_true", dest="cli", help="Run in CLI")
	(options, args) = parser.parse_args()
	if options.cli:
		gui = False
	else:
		gui = True
	window = IconApp(gui)
	gtk.main()

